using System;
using System.Text;
using System.IO;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;

class Program
{
	static int width=0,height=0;
	static int maxw=0,maxh=0,maxl=0;
	static string target="";
	static Dictionary<string, Bitmap> images= new Dictionary<string, Bitmap>();
	static List<string> tile = new List<string>();
	static int trans = -1;
	
	
	static bool ReadFile(string file) {
		/* ݒt@C̓ǂݍ */
		
		try {
			StreamReader sr = new StreamReader(file);
			string line = sr.ReadLine();
			string[] header = line.Split(',');
			
			if(header.Length < 3) {
				Console.WriteLine("Header malformed.");
				return false;
			}
			
			Int32.TryParse(header[0],out width);
			Int32.TryParse(header[1],out height);
			target = header[2];
			
			if (header.Length>3)
				trans = Convert.ToInt32(header[3], 16);
			
			while((line = sr.ReadLine()) != null) {
				if(line.StartsWith("--")) break;
				string[] data = line.Split(' ');
				if(data.Length<2 || data[0].Length!=1) {
					Console.WriteLine("Line malformed : {0}",line);
					return false;
				}
				Bitmap img = new Bitmap(data[1]);
				Console.WriteLine("Load image file:{0} Width:{1} Height:{2}",data[1],img.Width,img.Height);
				if(maxw < img.Width) maxw = img.Width;
				if(maxh < img.Height) maxh = img.Height;
				images[data[0]] = img;
			}
			
			/* ő̕E擾 */
			if (width==0) width = maxw;
			if (height==0) height = maxh;
			while((line = sr.ReadLine()) != null) {
				if(line.Length > maxl) maxl = line.Length;
				tile.Add(line);
			}
			
			while(tile.Count > 0 && tile[tile.Count-1].Length==0) {
				tile.RemoveAt(tile.Count-1);
			}
			
			if(maxl==0 || tile.Count==0) {
				Console.WriteLine("map data not found");
			return false;
			}
			
			
		} catch (FileNotFoundException e) {
			Console.WriteLine("File not Found : " + e.Message);
			return false;
		} catch (Exception e) {
			Console.WriteLine(e.Message);
			return false;
		}
		
		return true;
	}
	
	
	static void Main(string[] args){
		
		if(args.Length==0) {
			Console.WriteLine("usage : cgconcat.exe script");
			return;
		}
		
		if(!ReadFile(args[0]))
			return;
			
		Console.WriteLine("Width={0} Height={1} Outfile={2} Transcolor={3:x}",width,height,target,trans);
		
		/* bitmap쐬 */
		Bitmap outimg = new Bitmap(width * maxl, height * tile.Count, PixelFormat.Format32bppArgb);
		for(int y=0;y<height * tile.Count;y++) {
			for(int x=0;x<width * maxl;x++) {
				outimg.SetPixel(x,y,Color.FromArgb(0,0,0,0));
			}
		}
		
		Color transcol = Color.FromArgb(255, (trans & 0xFF0000)>>16, (trans & 0xFF00)>>8, (trans & 0xFF));
		for(int sy=0;sy<tile.Count;sy++) {
			string line = tile[sy];
			
			for(int sx=0;sx<maxl;sx++) {
				if(line.Length <= sx) continue;
				string key = line.Substring(sx,1);
				if(key==" ") continue;
				if(!images.ContainsKey(key)){
					Console.WriteLine("({0},{1}) : Skip: Key not found : {2}.",sx+1,sy+1,key);
					continue;
				}
				
				Bitmap bmp = images[key];
				for(int y=0;y<height;y++) {
					for(int x=0;x<width;x++) {
						Color col;
						if(y>=bmp.Height || x>=bmp.Width) {
							col = Color.FromArgb(0,0,0,0);
						}
						else {
							col = bmp.GetPixel(x,y);
							if (trans != -1 && col == transcol) col = Color.FromArgb(0,0,0,0);
						}
						outimg.SetPixel(sx*width+x,sy*height+y,col);
					}
				}
				
			}
		}
		
		try {
			outimg.Save(target);
		} catch (Exception e) {
			Console.WriteLine("Output Failed : " + e.Message);
		}
		
		
	}
}
